#!/usr/bin/python3

import subprocess
import shlex

from Gui.Gui import WenuxGUI
from PyQt5 import QtWidgets
import sys

if __name__ == "__main__":
    app = QtWidgets.QApplication([])
    subprocess.call(shlex.split(f"./bridge.sh setup {sys.argv[1]} {sys.argv[2]}"))
    application = WenuxGUI()
    application.show()
    sys.exit(app.exec())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'qt5.ui'
#
# Created by: PyQt5 UI code generator 5.13.2
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_WEnux(object):
    def setupUi(self, WEnux):
        WEnux.setObjectName("WEnux")
        WEnux.resize(983, 865)
        WEnux.setAcceptDrops(False)
        self.centralwidget = QtWidgets.QWidget(WEnux)
        self.centralwidget.setObjectName("centralwidget")
        self.submitButton = QtWidgets.QPushButton(self.centralwidget)
        self.submitButton.setGeometry(QtCore.QRect(850, 770, 104, 36))
        self.submitButton.setObjectName("submitButton")
        self.interfaceBox = QtWidgets.QComboBox(self.centralwidget)
        self.interfaceBox.setGeometry(QtCore.QRect(100, 30, 126, 37))
        self.interfaceBox.setObjectName("interfaceBox")
        self.interfaceLabel = QtWidgets.QLabel(self.centralwidget)
        self.interfaceLabel.setGeometry(QtCore.QRect(20, 40, 67, 19))
        self.interfaceLabel.setObjectName("interfaceLabel")
        self.directionLabel = QtWidgets.QLabel(self.centralwidget)
        self.directionLabel.setGeometry(QtCore.QRect(260, 40, 67, 19))
        self.directionLabel.setObjectName("directionLabel")
        self.debugBox = QtWidgets.QCheckBox(self.centralwidget)
        self.debugBox.setGeometry(QtCore.QRect(620, 780, 95, 25))
        self.debugBox.setObjectName("debugBox")
        self.outputBrowser = QtWidgets.QTextBrowser(self.centralwidget)
        self.outputBrowser.setGeometry(QtCore.QRect(40, 620, 881, 141))
        self.outputBrowser.setObjectName("outputBrowser")
        self.horizontalLayoutWidget = QtWidgets.QWidget(self.centralwidget)
        self.horizontalLayoutWidget.setGeometry(QtCore.QRect(40, 280, 881, 161))
        self.horizontalLayoutWidget.setObjectName("horizontalLayoutWidget")
        self.tcBox = QtWidgets.QHBoxLayout(self.horizontalLayoutWidget)
        self.tcBox.setContentsMargins(0, 0, 0, 0)
        self.tcBox.setObjectName("tcBox")
        self.reorderBox = QtWidgets.QGroupBox(self.horizontalLayoutWidget)
        self.reorderBox.setObjectName("reorderBox")
        self.formLayoutWidget_4 = QtWidgets.QWidget(self.reorderBox)
        self.formLayoutWidget_4.setGeometry(QtCore.QRect(10, 21, 243, 92))
        self.formLayoutWidget_4.setObjectName("formLayoutWidget_4")
        self.formLayout_5 = QtWidgets.QFormLayout(self.formLayoutWidget_4)
        self.formLayout_5.setContentsMargins(0, 0, 0, 0)
        self.formLayout_5.setObjectName("formLayout_5")
        self.reorderName = QtWidgets.QLabel(self.formLayoutWidget_4)
        self.reorderName.setStatusTip("")
        self.reorderName.setObjectName("reorderName")
        self.formLayout_5.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.reorderName)
        self.reorderValue = QtWidgets.QSpinBox(self.formLayoutWidget_4)
        self.reorderValue.setToolTip("")
        self.reorderValue.setMaximum(100)
        self.reorderValue.setSingleStep(1)
        self.reorderValue.setProperty("value", 0)
        self.reorderValue.setObjectName("reorderValue")
        self.formLayout_5.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.reorderValue)
        self.reorder_correlationName = QtWidgets.QLabel(self.formLayoutWidget_4)
        self.reorder_correlationName.setStatusTip("")
        self.reorder_correlationName.setObjectName("reorder_correlationName")
        self.formLayout_5.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.reorder_correlationName)
        self.reorder_correlationValue = QtWidgets.QSpinBox(self.formLayoutWidget_4)
        self.reorder_correlationValue.setToolTip("")
        self.reorder_correlationValue.setMaximum(100)
        self.reorder_correlationValue.setSingleStep(1)
        self.reorder_correlationValue.setProperty("value", 0)
        self.reorder_correlationValue.setObjectName("reorder_correlationValue")
        self.formLayout_5.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.reorder_correlationValue)
        self.tcBox.addWidget(self.reorderBox)
        self.rateBox = QtWidgets.QGroupBox(self.horizontalLayoutWidget)
        self.rateBox.setObjectName("rateBox")
        self.formLayoutWidget_13 = QtWidgets.QWidget(self.rateBox)
        self.formLayoutWidget_13.setGeometry(QtCore.QRect(9, 27, 271, 141))
        self.formLayoutWidget_13.setObjectName("formLayoutWidget_13")
        self.formLayout_15 = QtWidgets.QFormLayout(self.formLayoutWidget_13)
        self.formLayout_15.setContentsMargins(0, 0, 0, 0)
        self.formLayout_15.setObjectName("formLayout_15")
        self.netemRateName = QtWidgets.QLabel(self.formLayoutWidget_13)
        self.netemRateName.setStatusTip("")
        self.netemRateName.setObjectName("netemRateName")
        self.formLayout_15.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.netemRateName)
        self.netemRateBox = QtWidgets.QComboBox(self.formLayoutWidget_13)
        self.netemRateBox.setObjectName("netemRateBox")
        self.formLayout_15.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.netemRateBox)
        self.netemOverheadName = QtWidgets.QLabel(self.formLayoutWidget_13)
        self.netemOverheadName.setStatusTip("")
        self.netemOverheadName.setObjectName("netemOverheadName")
        self.formLayout_15.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.netemOverheadName)
        self.netemOverheadValue = QtWidgets.QSpinBox(self.formLayoutWidget_13)
        self.netemOverheadValue.setToolTip("")
        self.netemOverheadValue.setMinimum(-100)
        self.netemOverheadValue.setMaximum(100)
        self.netemOverheadValue.setSingleStep(1)
        self.netemOverheadValue.setProperty("value", 0)
        self.netemOverheadValue.setObjectName("netemOverheadValue")
        self.formLayout_15.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.netemOverheadValue)
        self.tcBox.addWidget(self.rateBox)
        self.reorderBox_3 = QtWidgets.QGroupBox(self.horizontalLayoutWidget)
        self.reorderBox_3.setObjectName("reorderBox_3")
        self.formLayoutWidget_11 = QtWidgets.QWidget(self.reorderBox_3)
        self.formLayoutWidget_11.setGeometry(QtCore.QRect(10, 21, 243, 92))
        self.formLayoutWidget_11.setObjectName("formLayoutWidget_11")
        self.formLayout_13 = QtWidgets.QFormLayout(self.formLayoutWidget_11)
        self.formLayout_13.setContentsMargins(0, 0, 0, 0)
        self.formLayout_13.setObjectName("formLayout_13")
        self.corruptName = QtWidgets.QLabel(self.formLayoutWidget_11)
        self.corruptName.setStatusTip("")
        self.corruptName.setObjectName("corruptName")
        self.formLayout_13.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.corruptName)
        self.corruptValue = QtWidgets.QSpinBox(self.formLayoutWidget_11)
        self.corruptValue.setToolTip("")
        self.corruptValue.setMaximum(100)
        self.corruptValue.setSingleStep(1)
        self.corruptValue.setProperty("value", 0)
        self.corruptValue.setObjectName("corruptValue")
        self.formLayout_13.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.corruptValue)
        self.corrupt_correlationName = QtWidgets.QLabel(self.formLayoutWidget_11)
        self.corrupt_correlationName.setStatusTip("")
        self.corrupt_correlationName.setObjectName("corrupt_correlationName")
        self.formLayout_13.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.corrupt_correlationName)
        self.corrupt_correlationValue = QtWidgets.QSpinBox(self.formLayoutWidget_11)
        self.corrupt_correlationValue.setToolTip("")
        self.corrupt_correlationValue.setMaximum(100)
        self.corrupt_correlationValue.setSingleStep(1)
        self.corrupt_correlationValue.setProperty("value", 0)
        self.corrupt_correlationValue.setObjectName("corrupt_correlationValue")
        self.formLayout_13.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.corrupt_correlationValue)
        self.tcBox.addWidget(self.reorderBox_3)
        self.horizontalLayoutWidget_2 = QtWidgets.QWidget(self.centralwidget)
        self.horizontalLayoutWidget_2.setGeometry(QtCore.QRect(40, 100, 881, 171))
        self.horizontalLayoutWidget_2.setObjectName("horizontalLayoutWidget_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.horizontalLayoutWidget_2)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.netemBox = QtWidgets.QGroupBox(self.horizontalLayoutWidget_2)
        self.netemBox.setObjectName("netemBox")
        self.formLayoutWidget = QtWidgets.QWidget(self.netemBox)
        self.formLayoutWidget.setGeometry(QtCore.QRect(10, 21, 253, 165))
        self.formLayoutWidget.setObjectName("formLayoutWidget")
        self.formLayout = QtWidgets.QFormLayout(self.formLayoutWidget)
        self.formLayout.setContentsMargins(0, 0, 0, 0)
        self.formLayout.setObjectName("formLayout")
        self.delayName = QtWidgets.QLabel(self.formLayoutWidget)
        self.delayName.setStatusTip("")
        self.delayName.setObjectName("delayName")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.delayName)
        self.jitterName = QtWidgets.QLabel(self.formLayoutWidget)
        self.jitterName.setStatusTip("")
        self.jitterName.setObjectName("jitterName")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.jitterName)
        self.jitterValue = QtWidgets.QSpinBox(self.formLayoutWidget)
        self.jitterValue.setToolTip("")
        self.jitterValue.setMaximum(500)
        self.jitterValue.setSingleStep(10)
        self.jitterValue.setProperty("value", 0)
        self.jitterValue.setObjectName("jitterValue")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.jitterValue)
        self.correlationName = QtWidgets.QLabel(self.formLayoutWidget)
        self.correlationName.setStatusTip("")
        self.correlationName.setObjectName("correlationName")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.correlationName)
        self.delay_jitter_corrValue = QtWidgets.QSpinBox(self.formLayoutWidget)
        self.delay_jitter_corrValue.setToolTip("")
        self.delay_jitter_corrValue.setMaximum(100)
        self.delay_jitter_corrValue.setSingleStep(1)
        self.delay_jitter_corrValue.setProperty("value", 0)
        self.delay_jitter_corrValue.setObjectName("delay_jitter_corrValue")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.delay_jitter_corrValue)
        self.delayValue = QtWidgets.QSpinBox(self.formLayoutWidget)
        self.delayValue.setToolTip("")
        self.delayValue.setMaximum(5000)
        self.delayValue.setSingleStep(10)
        self.delayValue.setProperty("value", 0)
        self.delayValue.setObjectName("delayValue")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.delayValue)
        self.horizontalLayout_2.addWidget(self.netemBox)
        self.lossBox_3 = QtWidgets.QGroupBox(self.horizontalLayoutWidget_2)
        self.lossBox_3.setObjectName("lossBox_3")
        self.formLayoutWidget_10 = QtWidgets.QWidget(self.lossBox_3)
        self.formLayoutWidget_10.setGeometry(QtCore.QRect(10, 21, 244, 92))
        self.formLayoutWidget_10.setObjectName("formLayoutWidget_10")
        self.formLayout_12 = QtWidgets.QFormLayout(self.formLayoutWidget_10)
        self.formLayout_12.setContentsMargins(0, 0, 0, 0)
        self.formLayout_12.setObjectName("formLayout_12")
        self.duplName = QtWidgets.QLabel(self.formLayoutWidget_10)
        self.duplName.setStatusTip("")
        self.duplName.setObjectName("duplName")
        self.formLayout_12.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.duplName)
        self.duplValue = QtWidgets.QSpinBox(self.formLayoutWidget_10)
        self.duplValue.setToolTip("")
        self.duplValue.setMaximum(100)
        self.duplValue.setSingleStep(1)
        self.duplValue.setProperty("value", 0)
        self.duplValue.setObjectName("duplValue")
        self.formLayout_12.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.duplValue)
        self.dupl_correlationName = QtWidgets.QLabel(self.formLayoutWidget_10)
        self.dupl_correlationName.setStatusTip("")
        self.dupl_correlationName.setObjectName("dupl_correlationName")
        self.formLayout_12.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.dupl_correlationName)
        self.dupl_correlationValue = QtWidgets.QSpinBox(self.formLayoutWidget_10)
        self.dupl_correlationValue.setToolTip("")
        self.dupl_correlationValue.setMaximum(100)
        self.dupl_correlationValue.setSingleStep(1)
        self.dupl_correlationValue.setProperty("value", 0)
        self.dupl_correlationValue.setObjectName("dupl_correlationValue")
        self.formLayout_12.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.dupl_correlationValue)
        self.horizontalLayout_2.addWidget(self.lossBox_3)
        self.lossBox = QtWidgets.QGroupBox(self.horizontalLayoutWidget_2)
        self.lossBox.setObjectName("lossBox")
        self.formLayoutWidget_3 = QtWidgets.QWidget(self.lossBox)
        self.formLayoutWidget_3.setGeometry(QtCore.QRect(10, 21, 243, 92))
        self.formLayoutWidget_3.setObjectName("formLayoutWidget_3")
        self.formLayout_4 = QtWidgets.QFormLayout(self.formLayoutWidget_3)
        self.formLayout_4.setContentsMargins(0, 0, 0, 0)
        self.formLayout_4.setObjectName("formLayout_4")
        self.lossName = QtWidgets.QLabel(self.formLayoutWidget_3)
        self.lossName.setStatusTip("")
        self.lossName.setObjectName("lossName")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.lossName)
        self.lossValue = QtWidgets.QSpinBox(self.formLayoutWidget_3)
        self.lossValue.setToolTip("")
        self.lossValue.setMaximum(100)
        self.lossValue.setSingleStep(1)
        self.lossValue.setProperty("value", 0)
        self.lossValue.setObjectName("lossValue")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.lossValue)
        self.loss_corellationName = QtWidgets.QLabel(self.formLayoutWidget_3)
        self.loss_corellationName.setStatusTip("")
        self.loss_corellationName.setObjectName("loss_corellationName")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.loss_corellationName)
        self.loss_corellationValue = QtWidgets.QSpinBox(self.formLayoutWidget_3)
        self.loss_corellationValue.setToolTip("")
        self.loss_corellationValue.setMaximum(100)
        self.loss_corellationValue.setSingleStep(1)
        self.loss_corellationValue.setProperty("value", 0)
        self.loss_corellationValue.setObjectName("loss_corellationValue")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.loss_corellationValue)
        self.horizontalLayout_2.addWidget(self.lossBox)
        self.resetButton = QtWidgets.QPushButton(self.centralwidget)
        self.resetButton.setGeometry(QtCore.QRect(730, 771, 104, 36))
        self.resetButton.setObjectName("resetButton")
        self.gridLayoutWidget = QtWidgets.QWidget(self.centralwidget)
        self.gridLayoutWidget.setGeometry(QtCore.QRect(190, 450, 291, 80))
        self.gridLayoutWidget.setObjectName("gridLayoutWidget")
        self.source = QtWidgets.QGridLayout(self.gridLayoutWidget)
        self.source.setContentsMargins(0, 0, 0, 0)
        self.source.setObjectName("source")
        self.sourcePort = QtWidgets.QLabel(self.gridLayoutWidget)
        self.sourcePort.setObjectName("sourcePort")
        self.source.addWidget(self.sourcePort, 1, 0, 1, 1)
        self.sourceNetworkLine = QtWidgets.QLineEdit(self.gridLayoutWidget)
        self.sourceNetworkLine.setObjectName("sourceNetworkLine")
        self.source.addWidget(self.sourceNetworkLine, 0, 1, 1, 1)
        self.sourcePortLine = QtWidgets.QLineEdit(self.gridLayoutWidget)
        self.sourcePortLine.setObjectName("sourcePortLine")
        self.source.addWidget(self.sourcePortLine, 1, 1, 1, 1)
        self.sourceNetwork = QtWidgets.QLabel(self.gridLayoutWidget)
        self.sourceNetwork.setObjectName("sourceNetwork")
        self.source.addWidget(self.sourceNetwork, 0, 0, 1, 1)
        self.gridLayoutWidget_2 = QtWidgets.QWidget(self.centralwidget)
        self.gridLayoutWidget_2.setGeometry(QtCore.QRect(490, 450, 291, 80))
        self.gridLayoutWidget_2.setObjectName("gridLayoutWidget_2")
        self.destination = QtWidgets.QGridLayout(self.gridLayoutWidget_2)
        self.destination.setContentsMargins(0, 0, 0, 0)
        self.destination.setObjectName("destination")
        self.destinationNetwork = QtWidgets.QLabel(self.gridLayoutWidget_2)
        self.destinationNetwork.setObjectName("destinationNetwork")
        self.destination.addWidget(self.destinationNetwork, 0, 0, 1, 1)
        self.destinationNetworkLine = QtWidgets.QLineEdit(self.gridLayoutWidget_2)
        self.destinationNetworkLine.setObjectName("destinationNetworkLine")
        self.destination.addWidget(self.destinationNetworkLine, 0, 1, 1, 1)
        self.destinationPort = QtWidgets.QLabel(self.gridLayoutWidget_2)
        self.destinationPort.setObjectName("destinationPort")
        self.destination.addWidget(self.destinationPort, 1, 0, 1, 1)
        self.destinationPortLine = QtWidgets.QLineEdit(self.gridLayoutWidget_2)
        self.destinationPortLine.setObjectName("destinationPortLine")
        self.destination.addWidget(self.destinationPortLine, 1, 1, 1, 1)
        self.gridLayoutWidget_3 = QtWidgets.QWidget(self.centralwidget)
        self.gridLayoutWidget_3.setGeometry(QtCore.QRect(490, 530, 291, 80))
        self.gridLayoutWidget_3.setObjectName("gridLayoutWidget_3")
        self.destination_3 = QtWidgets.QGridLayout(self.gridLayoutWidget_3)
        self.destination_3.setContentsMargins(0, 0, 0, 0)
        self.destination_3.setObjectName("destination_3")
        self.destinationNetwork_3 = QtWidgets.QLabel(self.gridLayoutWidget_3)
        self.destinationNetwork_3.setObjectName("destinationNetwork_3")
        self.destination_3.addWidget(self.destinationNetwork_3, 0, 0, 1, 1)
        self.destinationNetworkLineE = QtWidgets.QLineEdit(self.gridLayoutWidget_3)
        self.destinationNetworkLineE.setObjectName("destinationNetworkLineE")
        self.destination_3.addWidget(self.destinationNetworkLineE, 0, 1, 1, 1)
        self.destinationPort_3 = QtWidgets.QLabel(self.gridLayoutWidget_3)
        self.destinationPort_3.setObjectName("destinationPort_3")
        self.destination_3.addWidget(self.destinationPort_3, 1, 0, 1, 1)
        self.destinationPortLineE = QtWidgets.QLineEdit(self.gridLayoutWidget_3)
        self.destinationPortLineE.setObjectName("destinationPortLineE")
        self.destination_3.addWidget(self.destinationPortLineE, 1, 1, 1, 1)
        self.gridLayoutWidget_4 = QtWidgets.QWidget(self.centralwidget)
        self.gridLayoutWidget_4.setGeometry(QtCore.QRect(190, 530, 291, 80))
        self.gridLayoutWidget_4.setObjectName("gridLayoutWidget_4")
        self.source_3 = QtWidgets.QGridLayout(self.gridLayoutWidget_4)
        self.source_3.setContentsMargins(0, 0, 0, 0)
        self.source_3.setObjectName("source_3")
        self.sourcePort_3 = QtWidgets.QLabel(self.gridLayoutWidget_4)
        self.sourcePort_3.setObjectName("sourcePort_3")
        self.source_3.addWidget(self.sourcePort_3, 1, 0, 1, 1)
        self.sourceNetworkLineE = QtWidgets.QLineEdit(self.gridLayoutWidget_4)
        self.sourceNetworkLineE.setObjectName("sourceNetworkLineE")
        self.source_3.addWidget(self.sourceNetworkLineE, 0, 1, 1, 1)
        self.sourcePortLineE = QtWidgets.QLineEdit(self.gridLayoutWidget_4)
        self.sourcePortLineE.setObjectName("sourcePortLineE")
        self.source_3.addWidget(self.sourcePortLineE, 1, 1, 1, 1)
        self.sourceNetwork_3 = QtWidgets.QLabel(self.gridLayoutWidget_4)
        self.sourceNetwork_3.setObjectName("sourceNetwork_3")
        self.source_3.addWidget(self.sourceNetwork_3, 0, 0, 1, 1)
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setGeometry(QtCore.QRect(100, 470, 67, 19))
        self.label.setObjectName("label")
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setGeometry(QtCore.QRect(100, 560, 67, 19))
        self.label_2.setObjectName("label_2")
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        self.label_3.setGeometry(QtCore.QRect(590, 40, 131, 19))
        self.label_3.setObjectName("label_3")
        self.rulesetBox = QtWidgets.QComboBox(self.centralwidget)
        self.rulesetBox.setGeometry(QtCore.QRect(730, 32, 126, 37))
        self.rulesetBox.setObjectName("rulesetBox")
        self.rulesetBox.addItem("")
        self.rulesetBox.addItem("")
        self.directionBox = QtWidgets.QComboBox(self.centralwidget)
        self.directionBox.setGeometry(QtCore.QRect(350, 29, 126, 37))
        self.directionBox.setObjectName("directionBox")
        self.directionBox.addItem("")
        self.directionBox.addItem("")
        self.directionBox.addItem("")
        WEnux.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(WEnux)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 983, 29))
        self.menubar.setObjectName("menubar")
        WEnux.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(WEnux)
        self.statusbar.setObjectName("statusbar")
        WEnux.setStatusBar(self.statusbar)

        self.retranslateUi(WEnux)
        QtCore.QMetaObject.connectSlotsByName(WEnux)

    def retranslateUi(self, WEnux):
        _translate = QtCore.QCoreApplication.translate
        WEnux.setWindowTitle(_translate("WEnux", "WEnux"))
        self.submitButton.setText(_translate("WEnux", "Submit"))
        self.interfaceLabel.setText(_translate("WEnux", "Interface"))
        self.directionLabel.setText(_translate("WEnux", "Direction"))
        self.debugBox.setText(_translate("WEnux", "Debug"))
        self.reorderBox.setTitle(_translate("WEnux", "Reorder"))
        self.reorderName.setToolTip(_translate("WEnux", "Ratio of packet reordering in a stream"))
        self.reorderName.setText(_translate("WEnux", "Reordering"))
        self.reorderValue.setSuffix(_translate("WEnux", " %"))
        self.reorder_correlationName.setToolTip(_translate("WEnux", "Correlation factor for the reorder ratio"))
        self.reorder_correlationName.setText(_translate("WEnux", "Correlation"))
        self.reorder_correlationValue.setSuffix(_translate("WEnux", " %  "))
        self.rateBox.setTitle(_translate("WEnux", "Rate"))
        self.netemRateName.setToolTip(_translate("WEnux", "Overall delay for each packet in ms"))
        self.netemRateName.setText(_translate("WEnux", "Limit"))
        self.netemOverheadName.setToolTip(_translate("WEnux", "Overhead of packet in bytes - positive value can serve as emulation of higher link layer headers, negative value can artificially strip the Ethernet header and/or simulate a link layer header\n"
"compression scheme"))
        self.netemOverheadName.setText(_translate("WEnux", "Ovearhead"))
        self.netemOverheadValue.setSuffix(_translate("WEnux", " B"))
        self.reorderBox_3.setTitle(_translate("WEnux", "Corruption"))
        self.corruptName.setToolTip(_translate("WEnux", "Ratio of insertion of invalid packets to a stream"))
        self.corruptName.setText(_translate("WEnux", "Corruption"))
        self.corruptValue.setSuffix(_translate("WEnux", " %"))
        self.corrupt_correlationName.setToolTip(_translate("WEnux", "Correlation factor for the corruption ratio"))
        self.corrupt_correlationName.setText(_translate("WEnux", "Correlation"))
        self.corrupt_correlationValue.setSuffix(_translate("WEnux", " %  "))
        self.netemBox.setTitle(_translate("WEnux", "Delay"))
        self.delayName.setToolTip(_translate("WEnux", "Overall delay for each packet in ms"))
        self.delayName.setText(_translate("WEnux", "Delay"))
        self.jitterName.setToolTip(_translate("WEnux", "Jitter in ms"))
        self.jitterName.setText(_translate("WEnux", "Jitter"))
        self.jitterValue.setSuffix(_translate("WEnux", " ms"))
        self.correlationName.setToolTip(_translate("WEnux", "Correlation factor for the random jitter"))
        self.correlationName.setText(_translate("WEnux", "Correlation"))
        self.delay_jitter_corrValue.setSuffix(_translate("WEnux", " %  "))
        self.delayValue.setSuffix(_translate("WEnux", " ms"))
        self.lossBox_3.setTitle(_translate("WEnux", "Duplication"))
        self.duplName.setToolTip(_translate("WEnux", "Percentage of packet to be duplicated"))
        self.duplName.setText(_translate("WEnux", "Duplication"))
        self.duplValue.setSuffix(_translate("WEnux", " %"))
        self.dupl_correlationName.setToolTip(_translate("WEnux", "Correlation factor for the random loss ratio"))
        self.dupl_correlationName.setText(_translate("WEnux", "Correlation"))
        self.dupl_correlationValue.setSuffix(_translate("WEnux", " %  "))
        self.lossBox.setTitle(_translate("WEnux", "Loss"))
        self.lossName.setToolTip(_translate("WEnux", "Percentage of packet to be dropped"))
        self.lossName.setText(_translate("WEnux", "Loss"))
        self.lossValue.setSuffix(_translate("WEnux", " %"))
        self.loss_corellationName.setToolTip(_translate("WEnux", "Correlation factor for the random loss ratio"))
        self.loss_corellationName.setText(_translate("WEnux", "Correlation"))
        self.loss_corellationValue.setSuffix(_translate("WEnux", " %  "))
        self.resetButton.setText(_translate("WEnux", "Reset"))
        self.sourcePort.setText(_translate("WEnux", "Source Port"))
        self.sourceNetworkLine.setText(_translate("WEnux", "0/0, ::/0"))
        self.sourceNetwork.setText(_translate("WEnux", "Source Network"))
        self.destinationNetwork.setText(_translate("WEnux", "Destination Network"))
        self.destinationPort.setText(_translate("WEnux", "Destination Port"))
        self.destinationNetwork_3.setText(_translate("WEnux", "Destination Network"))
        self.destinationPort_3.setText(_translate("WEnux", "Destination Port"))
        self.destinationPortLineE.setText(_translate("WEnux", "22"))
        self.sourcePort_3.setText(_translate("WEnux", "Source Port"))
        self.sourcePortLineE.setText(_translate("WEnux", "22"))
        self.sourceNetwork_3.setText(_translate("WEnux", "Source Network"))
        self.label.setText(_translate("WEnux", "Include"))
        self.label_2.setText(_translate("WEnux", "Exclude"))
        self.label_3.setText(_translate("WEnux", "Predefined rulesets"))
        self.rulesetBox.setItemText(0, _translate("WEnux", "None"))
        self.rulesetBox.setItemText(1, _translate("WEnux", "Unreliable 100 Mb link"))
        self.directionBox.setItemText(0, _translate("WEnux", "Egress"))
        self.directionBox.setItemText(1, _translate("WEnux", "Ingress"))
        self.directionBox.setItemText(2, _translate("WEnux", "Bidirectional"))



import subprocess
import shlex
import logging


def call(command):
    logging.debug(command)
    subprocess.call(shlex.split(command))


def check_call(command):
    logging.debug(command)
    subprocess.check_call(shlex.split(command))


def run(command):
    logging.debug(command)
    return subprocess.run(shlex.split(command), stdout=subprocess.PIPE).stdout.decode('utf-8')



class TcCollection(list):
    @staticmethod
    def _operation_check(operation):
        _allowed_operation = ["add", "reset", "change"]

        if not (operation in _allowed_operation):
            raise Exception("Invalid operation parameter")
        else:
            return



from dataclasses import dataclass
from abc import abstractmethod
from Utils.Subprocess import check_call
from Tc.TcCollection import TcCollection


class TbfCmd(object):
    """Abstract class for Tbf commands

    Class implements abstract method for all tbf commands
    to specify

   """
    @abstractmethod
    def make_cmd(self):
        pass

    @staticmethod
    def format(to_prepend, value, to_append):
        if value:
            return f"{to_prepend} {value}{to_append}"
        else:
            return ""


@dataclass(eq=True, frozen=True)
class Tbf(TbfCmd):
    """Tbf parameter class

    Class handles the generation of partial command for tbf

    Class requires to have have parameters eq and frozen set
    to true - needed for set eq and hash to decide uniqueness of object,
    hash is generated by default if eq and frozen
    """
    _limit: int = 0
    _burst: int = 2000
    _latency: int = 20

    def make_cmd(self):
        _limit = self.format("", self._limit, "kbit")
        _burst = self.format("burst", self._burst, "kb")
        _latency = self.format("latency", self._latency, "ms")
        return f"rate {_limit} {_burst} {_latency}"


class TbfCollection(TcCollection):
    """Class for composing and running the tbf command

    All classes for tbf parameters are aggregated together
    into one big command and run at once
    """
    def __init__(self, nic):
        super().__init__()
        self._nic = nic

    def apply_cmd(self, operation):
        """Function that loops over all dataclasses and calls netem

        Dataclasses with their respective partial commands based
        on parameters are aggregated and main netem command is run
        """
        _used_dtcls = set()

        for dtcls in self:
            for param_val in dtcls.__dict__.values():
                if param_val:
                    _used_dtcls.add(dtcls)

        check_call(f"tc qdisc {operation} dev {self._nic} parent 1:3 handle 30: tbf "
                   + " ".join([x.make_cmd() for x in _used_dtcls]))




import logging
import datetime
import time

from Utils.Subprocess import call, check_call
from Tc.Tbf import TbfCollection, Tbf
from Tc.Filter import Filter, TcFilter
from Tc.Netem import NetemCollection, Duplicate, Delay, Loss, Reorder, Corrupt,Rate


class TcWrapper(object):
    """Wrapper around tc module

    Class functions as a wrapper before all tc related
    classes are called, handles their parameters and
    does basic initialization along with teardown

    """
    def __init__(self, **kwargs):
        for key in kwargs:
            setattr(self, key, kwargs[key])
        nic = self.nic
        self.nic = 'ifb1' if not self.direction == "Egress" else nic
        self.real_nic = nic

    def tbf(self, operation):
        """Control of tbf

        :param operation type of operation to be executed

        Function hands over appropriate parameters to tbf,
        creates the command and applies it
        """
        tbf = Tbf(self.limit,
                  self.burst,
                  self.latency)

        tbf_collection = TbfCollection(self.nic)
        tbf_collection.append(tbf)
        tbf_collection.apply_cmd(operation=operation)

    def netem(self, operation):
        """Control of netem

        :param operation type of operation to be executed

        Function hands over appropriate parameters to netem,
        creates the command and applies it
        """
        loss = Loss(self.loss_ratio,
                    self.loss_corr)

        dupl = Duplicate(self.dupl_ratio,
                         self.dupl_corr)

        delay = Delay(self.delay,
                      self.jitter,
                      self.delay_jitter_corr)

        reorder = Reorder(self.reorder_ratio,
                          self.reorder_corr)

        corrupt = Corrupt(self.corrupt_ratio)

        rate = Rate(self.limit,
                    self.overhead)

        netem_collection = NetemCollection(self.nic)
        netem_collection.extend([loss, dupl, delay, reorder, corrupt, rate])
        netem_collection.apply_cmd(operation=operation)

    def initialize(self):
        """Initialization of qdisc scheduler and filters

        Function decides what kind of qdiscs are required based on
        the direction of the emulation. Inclusion and exclusion
        filters are applied on top of the root qdisc to create
        a tree hierarchy

        """
        logging.info("Initializing and cleaning up previous configuration.")

        if not self.direction == "Egress":
            self.ingress_setup()

        call(f"tc qdisc del root dev {self.nic} ")
        check_call(f"tc qdisc add dev {self.nic} root handle 1: prio")

        exclude_filters, exclude_filters_ipv6 = Filter.generate_filters(self.exclude)
        TcFilter(exclude_filters, exclude_filters_ipv6, self.nic, "exclude")

        include_filters, include_filters_ipv6 = Filter.generate_filters(self.include)
        TcFilter(include_filters, include_filters_ipv6, self.nic, "include")

    def teardown(self):
        """Teardown of existing tc setup

        Function based on the direction of the emulation deletes
        approriate qdisc, filters and interfaces to
        end the emulation
        """
        if not self.direction == "Egress":
            call(f"tc filter del dev {self.real_nic} parent ffff: protocol ip prio 1")
            call(f"tc qdisc del dev {self.real_nic} ingress")
            call("ip link set dev ifb0 down")

        call(f"tc qdisc del root dev {self.nic}")
        logging.info("Network impairment emulation teardown complete.")

    def ingress_setup(self):
        """Function to setup the ingress datapath

        If ingress parameter is specified ifb module
        is loaded and ingress qdisc of real interface
        is replaced with ifb to be able to perform
        all tc related operations
        """
        check_call("modprobe ifb")
        check_call(f"ip link set dev {self.nic} up")
        call(f"tc qdisc del dev {self.real_nic} ingress ")
        check_call(f"tc qdisc replace dev {self.real_nic} ingress")
        check_call(f"tc filter replace dev {self.real_nic} parent ffff: protocol "
                   f"ip prio 1 u32 match u32 0 0 flowid 1:1 action mirred egress "
                   f"redirect dev {self.nic}")



from Gui.Qt5 import Ui_WEnux
import os
import subprocess
import traceback
import Gui.Logging
import logging

from Tc.TcWrapper import TcWrapper
from Utils.ResourceMonitor import resource_monitor
from PyQt5 import QtWidgets, QtCore, QtGui
from Utils.Subprocess import run
import socket
from PyQt5.QtWidgets import QMessageBox

known_limits = {
    "Modem / Dialup": 56,
    "ADSL Lite": 1500,
    "T1 / DS1": 1544,
    "E1 / E - carrier": 2048,
    "ADSL1": 4000,
    "Ethernet": 10000,
    "Wireless 802.11b": 11000,
    "ADSL2 +": 24000,
    "T3 / DS3": 44736,
    "Wireless 802.11g": 54000,
    "Fast Ethernet": 100000
}

args = {}


class WenuxGUI(QtWidgets.QMainWindow):
    """WenuxGUI is the main part of PyQt5 GUI usage and control of WEnux

    This class is used as setup for the graphical user interface,
    it renders the look of GUI with PyQt5 library and handles the argument
    parsing from the GUI.
    Arguments are validated and later used for the calls to WEnux core.
    Class includes setup of logging, output and control of Qt related objects.

   """

    def __init__(self):
        super(WenuxGUI, self).__init__()
        self.ui = Ui_WEnux()
        self.ui.setupUi(self)

        stdout = Gui.Logging.OutputWrapper(self, True)
        stdout.outputWritten.connect(self.handle_output)
        stderr = Gui.Logging.OutputWrapper(self, False)
        stderr.outputWritten.connect(self.handle_output)
        self.terminal = self.ui.outputBrowser

        self.fill_boxes()
        self.ui.submitButton.clicked.connect(self.submit_clicked)
        self.ui.resetButton.clicked.connect(self.reset_clicked)

        if os.geteuid() != 0:
            logging.error("You need root permissions to run WEnux!")
            exit(3)

    def closeEvent(self, event):
        """Function overrides the default close button UI event

        As user can prematurely quit the application teardown of different
        objects is required. After teardown is done, exit event can be accepted.

       """
        try:
            tc = TcWrapper(**args)
            tc.teardown()
        except:
            logging.info("Quitting the application without configuration.")
        finally:
            run(f"./bridge.sh teardown")
            event.accept()

    def handle_output(self, text, stdout):
        """Setup of text output

        Text printed on standard output either through logging module or printing
        is redirected to the main window serving as a graphical console.

       """
        color = self.terminal.textColor()
        self.terminal.setTextColor(color if stdout else QtCore.Qt.red)
        self.terminal.insertPlainText(text)
        self.terminal.setTextColor(color)

    def fill_boxes(self):
        """Filling out of dynamic graphical elements

        Based on the system used with different interfaces the combo box
        with all interfaces has to be filled out each time application is run.
        Limit selection can be done with predefined limits, which are formatted.

       """
        self.ui.interfaceBox.addItem("test_bridge")
        for interface in os.listdir('/sys/class/net/'):
            self.ui.interfaceBox.addItem(interface)

        for type, limit in known_limits.items():
            if limit > 1000:
                unit = "Mbit/s"
                limit /= 1000
            else:
                unit = "kbit/s"
            self.ui.netemRateBox.addItem(f"{type} - {limit} {unit}")
            self.ui.netemRateBox.setCurrentIndex(5)

    def load_args(self):
        """Parser for arguments with basic checks

        Function inspects each Qt element where could be useful parameters
        for the Tc module. Arguments are checked, and basically sanitized
        to be appropriate types for later use.

        :return bool: result of argument loading, checking and sanitizing
       """
        output = {}

        debug = self.ui.debugBox.isChecked()
        Gui.Logging.set_logging(debug)

        args["nic"] = str(self.ui.interfaceBox.currentText())
        args["delay"] = self.ui.delayValue.value()
        args["jitter"] = self.ui.jitterValue.value()
        args["delay_jitter_corr"] = self.ui.delay_jitter_corrValue.value()
        args["overhead"] = self.ui.netemOverheadValue.value()
        args["limit"] = list(known_limits.values())[self.ui.netemRateBox.currentIndex()]
        args["direction"] = self.ui.directionBox.currentText()
        args["loss_ratio"] = self.ui.lossValue.value()
        args["loss_corr"] = self.ui.loss_corellationValue.value()
        args["dupl_ratio"] = self.ui.duplValue.value()
        args["dupl_corr"] = self.ui.dupl_correlationValue.value()
        args["reorder_ratio"] = self.ui.reorderValue.value()
        args["reorder_corr"] = self.ui.reorder_correlationValue.value()
        args["corrupt_ratio"] = self.ui.corruptValue.value()
        args["corrupt_corr"] = self.ui.corrupt_correlationValue.value()
        args["include"] = []
        args["exclude"] = []
        ret_code = self.args_check()

        for k, v in args.items():
            if v is 0:
                continue
            else:
                output[k] = v

        logging.info(output)
        return ret_code

    def submit_clicked(self):
        if not self.load_args():
            wenux_submit()
        else:
            pass

    def reset_clicked(self):
        wenux_reset()

    def get_args(self):
        return args

    def args_check(self):
        """Argument checker and sanitizer for later use in Tc module

        Parameters that arrive from the GUI frontend are checked if they contain
        correct information, such as IP addresses, ports.
        If they are valid, they are sanitized for later use

        :return bool: result of argument checking and sanitizing

       """
        for item in [self.ui.destinationNetworkLine, self.ui.destinationNetworkLineE,
                     self.ui.sourceNetworkLine, self.ui.sourceNetworkLineE,
                     self.ui.sourcePortLine, self.ui.sourcePortLineE,
                     self.ui.destinationPortLineE, self.ui.destinationPortLine
                     ]:
            if "destination" in item.objectName():
                if "Port" in item.objectName():
                    prefix = "dport="
                else:
                    prefix = "dst="
            else:
                if "Port" in item.objectName():
                    prefix = "sport="
                else:
                    prefix = "src="

            if item.text():
                for attr in item.text().split(","):
                    try:
                        attr = attr.strip()
                    except AttributeError:
                        pass

                    if not validate_addr(attr):
                        if "E" in item.objectName():
                            args["exclude"].append(f"{prefix}" + attr)
                        else:
                            args["include"].append(f"{prefix}" + attr)
                    elif not validate_port(item.text()):
                        if "E" in item.objectName():
                            args["exclude"].append(f"{prefix}" + attr)
                        else:
                            args["include"].append(f"{prefix}" + attr)
                    else:
                        return 1
                    continue


# WEnux

def wenux_submit():
    """Main call to the Tc module that controls the emulation

    TcWrapper that gathers the arguments and controls the emulation
    load the arguments, initializes the setup according to parameters
    and calls appropriate tc module for the emulation.
    If emulation fails, it tears down the setup and quits the application

   """
    try:
        tc = TcWrapper(**args)
        tc.initialize()

        tc.netem("add")
        # tc.tbf("add")

    except subprocess.CalledProcessError:
        traceback.print_exc()
        tc.teardown()
        exit(2)


def wenux_reset():
    try:
        tc = TcWrapper(**args)
        tc.teardown()
    except:
        pass


def popup_error(err_type, text):
    """Popup QMessageBox in case of invalid arguments

    When invalid arguments are presented in the GUI by user,
    popup window shows up and doesn't let user continue

   """
    msg = QMessageBox()
    msg.setWindowTitle(f"Invalid {err_type}")
    msg.setText(text)
    msg.setIcon(QMessageBox.Critical)
    msg.exec_()


def validate_port(port):
    print(port)
    if not 0 <= int(port) <= 65535:
        popup_error("port", "The port you have entered is not a valid port number!")
        return 1
    return 0


def validate_addr(addr):
    """Validation of network address to be used in inclusion or exclusion

    :param addr is first checked whether it is in a network format or
    already in IP addresss notation. It separates mask and address or
    if needed adds a mask.

    Mask is verified based on the IP family it belongs to and appropriate
    text to network structure function is called to verify whether
    the IP adress is correct, if the exception is raised adress is invalid

    :return bool: state of validation

   """
    try:
        if "/" in addr:
            mask = addr.partition("/")[2]
            addr = addr.partition("/")[0]
        else:
            mask = 32

        if not ":" in addr:
            if not 0 <= int(mask) <= 32:
                raise socket.error
            socket.inet_aton(addr)
        else:
            if not 0 <= int(mask) <= 128:
                raise socket.error
            socket.inet_pton(socket.AF_INET6, addr)
    except socket.error:
        popup_error("IP address", "The IP you have entered is not a valid address!")
        return 1
    return 0



import sys
from PyQt5 import QtCore
import logging

def set_logging(debug):
    """Setup of run wide logging options

    :param debug is a bool value of debug argument

    Level of logging is setup based on the argument and logger is created
    with basic config

   """
    level = (logging.INFO, logging.DEBUG)[debug]
    if logging:
        logging.getLogger().setLevel(level)
    else:
        logging.basicConfig(level=level,
                            format="%(asctime)s %(levelname)-8s %(message)s",
                            datefmt="%Y-%m-%d %H:%M:%S")

class OutputWrapper(QtCore.QObject):
    """Output redirect class

    This class overrides the functionality of OutputWrapper to allow
    user to use logging module and output it to any Qt object that supports
    text input

   """
    outputWritten = QtCore.pyqtSignal(object, object)

    def __init__(self, parent=None, stdout=True):
        QtCore.QObject.__init__(self, parent)
        if stdout:
            self._stream = sys.stdout
            sys.stdout = self
        else:
            self._stream = sys.stderr
            sys.stderr = self
        self._stdout = stdout

    def write(self, text):
        self._stream.write(text)
        self.outputWritten.emit(text, self._stdout)

    def __getattr__(self, name):
        return getattr(self._stream, name)

    def __del__(self):
        try:
            if self._stdout:
                sys.stdout = self._stream
            else:
                sys.stderr = self._stream
        except AttributeError:
            pass



import logging

from dataclasses import dataclass
from abc import abstractmethod

from Utils.Subprocess import check_call


class Filter(object):
    """Handling of Tc Filter module

    Class handles the generation of filters required for the emulation.
    It serves as a parent to the TcFilter class as well.

   """
    _flowid = ""
    @abstractmethod
    def _apply_filter(self, filter_list, ip_version, priority):
        pass

    @staticmethod
    def generate_filters(filter_list):
        """Generation of strings for the tc filter command

        :param filter_list is a sanitized list of parameters from the GUI
        either include or exclude

        Based on the type of of IP family, direction and type of port
        supplied list is parsed and tokenized.
        After function figures out which type of token is in the list
        it creates appropriate filter string list

        :returns filter_strings, filter_strings_ipv6 are lists of
        partial filter commands according to the IP family

       """
        filter_strings = []
        filter_strings_ipv6 = []

        for tc_filter in filter_list:
            filter_tokens = tc_filter.split(",")

            try:
                filter_string = ""
                filter_string_ipv6 = ""

                for token in filter_tokens:
                    token_split = token.split("=")
                    key = token_split[0]
                    value = token_split[1]

                    if key == "src" or key == "dst":
                        if "::" in value:
                            filter_string_ipv6 += f"match ip6 {key} {value} "
                        else:
                            filter_string += f"match ip {key} {value} "
                    else:
                        filter_string += f"match ip {key} {value} "
                        filter_string_ipv6 += f"match ip6 {key} {value} "
                    if key == "sport" or key == "dport":
                        filter_string += "0xffff "
                        filter_string_ipv6 += "0xffff "

            except IndexError:
                logging.error("Error: Invalid filter parameters")

            if filter_string:
                filter_strings.append(filter_string)
            if filter_string_ipv6:
                filter_strings_ipv6.append(filter_string_ipv6)

        return filter_strings, filter_strings_ipv6


@dataclass
class TcFilter(Filter):
    """Manipulation with tc filter and application of filters

    Class handles the previously generated partial filters commands
    and creates the full command to be run based on the type of
    filter - either inclusion or exclusion.
   """
    _ip: str = None
    _ipv6: str = None
    _nic: str = None
    _type: str = None
    _priority: int = 0

    def __post_init__(self):
        """Function that allows dataclass to be fully executed on creation

        Class parameters are checked for integrity, variables are set
        accordingly and filter commands are executed upon prio qdisc
        for both IP families if supplied.
       """
        self._type_check(self._type)

        if self._type == "exclude":
            self._flowid = "1:2"
            self._priority = 1
            logging.debug(f"Excluding the following from network impairment:")
        else:
            self._flowid = "1:3"
            self._priority = 3
            logging.debug(f"Including the following for network impairment:")

        self._apply_filter(self._ip, "ip", self._priority)
        self._apply_filter(self._ipv6, "ipv6", self._priority+1)

    def _apply_filter(self, filter_list, ip_version, priority):
        for rule in filter_list:
            filter_cmd = f"tc filter add dev {self._nic} protocol {ip_version} " \
                         f"parent 1:0 prio {priority} u32 {rule}flowid {self._flowid}"
            check_call(filter_cmd)

    def _type_check(self, type):
        _allowed_types = ["include", "exclude"]
        if not (type in _allowed_types):
            raise Exception("Invalid type parameter")
        else:
            return



from dataclasses import dataclass
from abc import abstractmethod
from Utils.Subprocess import check_call
from Tc.TcCollection import TcCollection


class NetemCmd(object):
    """Abstract class for Netem commands

    Class implements abstract method for all netem commands
    to specify

   """
    @abstractmethod
    def make_cmd(self):
        pass

    @staticmethod
    def format(value, to_append):
        if value:
            return f"{value}{to_append}"
        else:
            return ""



@dataclass(eq=True, frozen=True)
class Loss(NetemCmd):
    """Netem Loss parameter class

    Class handles the generation of partial command for netem
    parameter - loss

    Class requires to have have parameters eq and frozen set
    to true - needed for set eq and hash to decide uniqueness of object,
    hash is generated by default if eq and frozen

    """
    _ratio: float = 0.0
    _correlation: float = 0.0

    def make_cmd(self):
        _ratio = self.format(self._ratio, "%")
        _correlation = self.format(self._correlation, "%")
        return f"loss {_ratio} {_correlation}"


@dataclass(eq=True, frozen=True)
class Duplicate(NetemCmd):
    """Netem Duplicate parameter class

    Class handles the generation of partial command for netem
    parameter - duplicate

    Class requires to have have parameters eq and frozen set
    to true - needed for set eq and hash to decide uniqueness of object,
    hash is generated by default if eq and frozen
    """
    _ratio: float = 0.0
    _correlation: float = 0.0

    def make_cmd(self):
        _ratio = self.format(self._ratio, "%")
        _correlation = self.format(self._correlation, "%")
        return f"duplicate {_ratio} {_correlation}"


@dataclass(eq=True, frozen=True)
class Delay(NetemCmd):
    """Netem Delay parameter class

    Class handles the generation of partial command for netem
    parameter - delay

    Class requires to have have parameters eq and frozen set
    to true - needed for set eq and hash to decide uniqueness of object,
    hash is generated by default if eq and frozen
    """
    _time: int = 0
    _jitter: int = 0
    _correlation: float = 0.0

    def make_cmd(self):
        _time = self.format(self._time, "ms")
        _jitter = self.format(self._jitter, "ms")
        _correlation = self.format(self._correlation, "%")

        return f"delay {_time} {_jitter} {_correlation}"


@dataclass(eq=True, frozen=True)
class Reorder(NetemCmd):
    """Netem Reorder parameter class

    Class handles the generation of partial command for netem
    parameter - reorder

    Class requires to have have parameters eq and frozen set
    to true - needed for set eq and hash to decide uniqueness of object,
    hash is generated by default if eq and frozen
    """
    _ratio: float = 0.0
    _correlation: float = 0.0

    def make_cmd(self):
        _ratio = self.format(self._ratio, "%")
        _correlation = self.format(self._correlation, "%")

        return f"reorder {_ratio} {_correlation}"


@dataclass(eq=True, frozen=True)
class Corrupt(NetemCmd):
    """Netem Corrupt parameter class

    Class handles the generation of partial command for netem
    parameter - corrupt

    Class requires to have have parameters eq and frozen set
    to true - needed for set eq and hash to decide uniqueness of object,
    hash is generated by default if eq and frozen
    """
    _ratio: float = 0.0
    _correlation: float = 0.0

    def make_cmd(self):
        _ratio = self.format(self._ratio, "%")
        _correlation = self.format(self._correlation, "%")

        return f"corrupt {_ratio} {_correlation}"


@dataclass(eq=True, frozen=True)
class Rate(NetemCmd):
    """Netem Rate parameter class

    Class handles the generation of partial command for netem
    parameter - rate

    Class requires to have have parameters eq and frozen set
    to true - needed for set eq and hash to decide uniqueness of object,
    hash is generated by default if eq and frozen
    """
    _rate: int = 0
    _overhead: int = 0

    def make_cmd(self):
        _rate = self.format(self._rate, "kbit")
        _overhead = self.format(self._overhead, "")

        return f"rate {_rate} {_overhead}"


class NetemCollection(TcCollection):
    """Class for composing and running the netem command

    All classes for netem parameters are aggregated together
    into one big command and run at once
    """
    def __init__(self, nic):
        super().__init__()
        self._nic = nic

    def apply_cmd(self, operation):
        """Function that loops over all dataclasses and calls netem

        Dataclasses with their respective partial commands based
        on parameters are aggregated and main netem command is run
        """
        _used_dtcls = set()

        self._operation_check(operation)

        for dtcls in self:
            for param_val in dtcls.__dict__.values():
                if param_val:
                    _used_dtcls.add(dtcls)

        # generate empty cmd to reset the tc qdisc device
        if operation == "reset":
            _used_dtcls = set()
            operation = "change"

        check_call(f"tc qdisc {operation} dev {self._nic} parent 1:3 handle 30: netem "
                   + " ".join([x.make_cmd() for x in _used_dtcls]))



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="wenux",  # Replace with your own username
    version="1.0.3",
    author="Jozef Urbanovský",
    author_email="jurbanov@redhat.com",
    description="WAN emulator for Linux",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Axonis/DP",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: POSIX :: Linux",
    ],
    python_requires='>=3.7',
)



from Utils.Subprocess import run
from time import sleep


# TODO resource monitor

def resource_monitor():
    last_idle = last_total = 0
    while True:
        with open('/proc/stat') as cpu:
            fields = [float(column) for column in cpu.readline().strip().split()[1:]]
        idle, total = fields[3], sum(fields)
        idle_delta, total_delta = idle - last_idle, total - last_total
        last_idle, last_total = idle, total
        cpu_stat = 100.0 * (1.0 - idle_delta / total_delta)

        print('Overall CPU Usage: %5.1f%%' % cpu_stat, end='\r')
        sleep(1)


def memory():
    with open('/proc/meminfo', 'r') as mem:
        tmp = 0
        for i in mem:
            sline = i.split()
            if str(sline[0]) == 'MemTotal:':
                total = int(sline[1])
            elif str(sline[0]) in ('MemFree:', 'Buffers:', 'Cached:'):
                tmp += int(sline[1])
        free = tmp
        mem_stat = int(total) - int(free)

    print(f"Memory usage: {mem_stat}")
